from copy import copy

from pymongo import MongoClient
from mymongo.constants import MongoConstants


class Mongodb:
    def __init__(self, connection_uri: str) -> None:
        super().__init__()
        self._client = MongoClient(connection_uri, serverSelectionTimeoutMS=5000)
        self._db = self._client[MongoConstants.ADMIN_DB]
        self._users_col = self._db[MongoConstants.SYSTEM_USERS_COL]

        self.users = None
        self.dbs = self._get_database_names()
        self.get_users()

    def create_user(self, name: str, roles: list, password: str = MongoConstants.DEFAULT_PASSWORD):
        res = self._db.command('createUser', name, pwd=password, roles=roles)
        self.get_users()
        return res

    def drop_user(self, username: str) -> dict:
        res = self._db.command('dropUser', username)
        self.get_users()
        return res

    def change_password(self, username: str, new_password: str) -> dict:
        return self._db.command('updateUser', username, pwd=new_password)

    def add_roles(self, username: str, new_roles: list) -> dict:
        user_roles = self._get_user_roles(username)
        c_user_roles = copy(user_roles)

        for new_role in new_roles:
            if new_role not in user_roles:
                user_roles.append(new_role)

        if c_user_roles == user_roles:
            return {}  # Nothing changed

        res = self._db.command('updateUser', username, roles=user_roles)
        self.get_users()
        return res

    def remove_roles(self, username: str, roles: list) -> dict:
        user_roles = self._get_user_roles(username)
        c_user_roles = copy(user_roles)

        for role in roles:
            if role in user_roles:
                user_roles.remove(role)

        if user_roles == c_user_roles:
            return {}

        res = self._db.command('updateUser', username, roles=user_roles)
        self.get_users()
        return res

    def get_users(self, db_name: str = MongoConstants.ADMIN_DB) -> None:
        _filter = {'db': db_name}
        _projection = {'_id': 0, 'user': 1, 'roles': 1}
        users = list(self._users_col.find(filter=_filter, projection=_projection))
        result = {item['user']: item['roles'] for item in users}
        self.users = result

    def _get_user_roles(self, username: str) -> list:
        user_info = self._users_col.find_one(filter={'user': username})
        return user_info['roles']

    def _get_database_names(self) -> list:
        return self._client.list_database_names()




if __name__ == '__main__':
    connection_url = "mongodb://dev:local@loclhost:27017"
    db = Mongodb(connection_url)
    # print(db.create_user('mmmdawn', [{'role': 'root', 'db': 'admin'}], '123123123'))
    print(db.users)



from . import Mongodb
from . import DB_ROLES, ALL_DB_ROLES, SUPERUSER_ROLES
from . import get_roles_msg, clear

from InquirerPy import inquirer
from InquirerPy.base.control import Choice, Separator


class App:
    def __init__(self):
        self.db = None

    def connect(self):
        connection_uri = inquirer.text(message='Connection URI:').execute()
        self.db: Mongodb = Mongodb(connection_uri)

    def home(self) -> None:
        while True:
            clear()
            choice = inquirer.fuzzy(
                message="Select:",
                choices=[
                    Choice(value=None, name="Create new account"),
                    'Exit',
                    *self.db.users,
                ],
                default=None,
            ).execute()
            clear()

            if choice:
                if choice == 'Exit':
                    print('\nBye\n')
                    input('\nPress any key to continue . . .')
                    break

                username = choice
                roles = self.db.users[username]
                print(get_roles_msg(username, roles))
                action = inquirer.select(
                    message='Choose action:',
                    choices=[
                        'Add role',
                        'Remove role',
                        Separator(),
                        'Change password',
                        'Drop user',
                        Separator(),
                        'Back'
                    ]
                ).execute()

                if action == 'Change password':
                    print(self.change_password(username))
                    input('\nPress any key to continue . . .')
                if action == 'Add role':
                    print(self.add_role(username))
                    input('\nPress any key to continue . . .')
                if action == 'Remove role':
                    print(self.remove_role(username))
                    input('\nPress any key to continue . . .')
                if action == 'Drop user':
                    print(self.drop_user(username))
                    input('\nPress any key to continue . . .')
                if action == 'Back':
                    continue
            else:
                print(self.create_user())
                input('\nPress any key to continue . . .')

    def create_user(self):
        try_time = 1
        password = ''
        username = ''
        while try_time <= 3:
            username = inquirer.text(message='Username:').execute()
            if username in self.db.users:
                print('\nUser already exists\n')
                try_time += 1
                continue

            password = inquirer.secret(message='Password:').execute()
            confirm_password = inquirer.secret(message='Confirm password:').execute()
            if password != confirm_password:
                print('\nNot match\n')
                try_time += 1
                continue
            else:
                break

        if try_time > 3:
            return "Cancelled"

        roles = self.get_roles()
        confirm = inquirer.confirm(
            message=f'Confirm add new user: {username} with role(s): {roles}',
            default=True
        ).execute()

        if confirm:
            return self.db.create_user(username, roles, password)
        else:
            return "Cancelled"

    def change_password(self, username: str):
        try_time = 1
        password = ''
        while try_time <= 3:
            password = inquirer.secret(message='New password:').execute()
            confirm_password = inquirer.secret(message='Confirm new password:').execute()
            if password != confirm_password:
                print('\nNot match\n')
                try_time += 1
                continue
            else:
                break

        if try_time > 3:
            return "Cancelled"
        return self.db.change_password(username, password)

    def get_roles(self):
        role = inquirer.select(
            message='Select role:',
            choices=[*DB_ROLES, Separator(), *ALL_DB_ROLES, Separator(), *SUPERUSER_ROLES]
        ).execute()

        if role in DB_ROLES:
            dbs = inquirer.select(message='Select database(s)', choices=self.db.dbs, multiselect=True).execute()
            new_roles = [{'role': role, 'db': db} for db in dbs]
        else:
            new_roles = [role]
        return new_roles

    def add_role(self, username: str):
        new_roles = self.get_roles()
        confirm = inquirer.confirm(message=f'Confirm add new role(s): {new_roles}', default=True).execute()
        if confirm:
            return self.db.add_roles(username, new_roles)
        else:
            return "Cancelled"

    def remove_role(self, username: str):
        roles = inquirer.select(
            message='Select role(s) to remove:',
            choices=[str(role) for role in self.db.users[username]],
            multiselect=True
        ).execute()

        roles = [eval(role) for role in roles]

        confirm = inquirer.confirm(message=f'Confirm remove role(s): {roles}', default=False).execute()

        if confirm:
            return self.db.remove_roles(username, roles)
        else:
            return "Cancelled"

    def drop_user(self, username: str):
        confirm = inquirer.confirm(message=f'Confirm drop user "{username}"', default=False).execute()
        if confirm:
            return self.db.drop_user(username)
        else:
            return "Cancelled"

    def run(self):
        self.connect()
        self.home()



from os import system, name


def get_roles_msg(username: str, roles: list):
    msg = f'Username: {username}\n' \
          f'Roles:'
    for role in roles:
        msg += f"\n\t{role['role']} - {role['db']}"
    msg += "\n------------------------------------------------------------------------------\n"
    return msg


def clear():
    if name == 'nt':
        _ = system('cls')
    else:
        _ = system('clear')



class MongoConstants:
    ADMIN_DB = 'admin'
    SYSTEM_USERS_COL = 'system.users'
    DEFAULT_PASSWORD = '123456'


DB_ROLES = [
    'read',
    'readWrite',
    'dbAdmin',
    'userAdmin',
    'dbOwner'
]

ALL_DB_ROLES = [
    'readAnyDatabase',
    'readWriteAnyDatabase',
    'userAdminAnyDatabase',
    'dbAdminAnyDatabase'
]

SUPERUSER_ROLES = [
    'root'
]



import setuptools

setuptools.setup(
    name="mongo-user-manager",
    version="0.0.1",
    author="Dawn",
    author_email="congminh292k@gmail.com",
    description="MongoDB utilities",
    keywords="mongodb user-management mongodb-role",
    license="GNU",
    url="https://github.com/mmmdawn/MongoUserManager",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Operating System :: POSIX :: Linux",
        "Topic :: Database"
    ],
    dependencies=[
        "inquirerpy ~= 0.3.4",
        "pymongo ~= 4.3.2",
    ],
    packages=["mymongo", "mymongo.database", "mymongo.utils"],
    entry_points={"console_scripts": ["mun=mymongo.__main__:main"]},
    python_requires=">=3.8",
)
